import { toRect, ensureEnum, brushToString, stringToBrush, ensureBool, initializePropertiesFromCss, NamePatcher, toSpinal, enumToString, getAllPropertyNames, fromSpinal } from "./componentUtil";
import { WebComponentRenderer, PortalManager } from "./web-component-renderer";
import { delegateCombine, delegateRemove } from "./type";
import { RegisterElementHelper } from "./module-manager";
import { Popup } from "./Popup";
import { IgcOnPopupEventArgs } from './igc-on-popup-event-args';
import { IgcOnClosedEventArgs } from './igc-on-closed-event-args';
import { IgcFocusEventArgs } from './igc-focus-event-args';
import { PopupAnimationType_$type } from "./PopupAnimationType";
import { PopupPointerPosition_$type } from "./PopupPointerPosition";
import { IgcHTMLElement } from "./igc-html-element";
import { IgcPopupMeasuringContentSizeEventArgs } from './igc-popup-measuring-content-size-event-args';
export let IgcPopupComponent = /*@__PURE__*/ (() => {
    class IgcPopupComponent extends IgcHTMLElement {
        set height(value) {
            this._height = value;
            this.style.height = value;
        }
        get height() {
            return this._height;
        }
        set width(value) {
            this._width = value;
            this.style.width = value;
        }
        get width() {
            return this._width;
        }
        constructor() {
            super();
            this._implementation = null;
            this._disconnected = false;
            this._settingAttributes = false;
            this._attached = false;
            this._queuedSetAttributes = [];
            this._updatingFromAttribute = false;
            this.__p = null;
            this._hasUserValues = new Set();
            this._stylingContainer = null;
            this._stylingParent = null;
            this._inStyling = false;
            this._onClosed = null;
            this._onClosed_wrapped = null;
            this._onPopup = null;
            this._onPopup_wrapped = null;
            this._popupGotFocus = null;
            this._popupGotFocus_wrapped = null;
            this._popupLostFocus = null;
            this._popupLostFocus_wrapped = null;
            this._measuringContentSize = null;
            this._measuringContentSize_wrapped = null;
            if (this._styling) {
                NamePatcher.ensureStylablePatched(Object.getPrototypeOf(this));
            }
            this._implementation = this.createImplementation();
            this._portalManager = new PortalManager("popupContent");
            this._webComponentRenderer = new WebComponentRenderer(this, document, true, {}, this._portalManager);
            this._mainDiv = this._webComponentRenderer.createElement("div");
            this._webComponentRenderer.updateRoot(this._mainDiv);
            this.i.provideRenderer(this._webComponentRenderer);
            let mut = new MutationObserver((list) => {
                for (var mutation of list) {
                    if (mutation.type == 'childList') {
                        this._onChildrenChanged();
                    }
                }
            });
            mut.observe(this, {
                childList: true
            });
        }
        _onChildrenChanged() {
            var children = [];
            for (var i = 0; i < this.childNodes.length; i++) {
                var child = this.childNodes[i];
                if (child !== this._mainDiv.getNativeElement()) {
                    children.push(child);
                }
            }
            this._portalManager.onChildrenChanged(children);
        }
        // supports angular themes or custom properties set in CSS
        updateStyle() {
            this._styling(this, this);
        }
        destroy() {
            this._implementation.destroy();
            this._webComponentRenderer.destroy();
        }
        createImplementation() {
            return new Popup();
        }
        get i() {
            return this._implementation;
        }
        disconnectedCallback() {
            this._disconnected = true;
        }
        connectedCallback() {
            if (this._disconnected) {
                this._disconnected = false;
                return;
            }
            this.classList.add("ig-popup");
            this.classList.add("igr-popup");
            this.appendChild(this._mainDiv.getNativeElement());
            this._attached = true;
            this.style.display = "block";
            this.style.height = this._height;
            this.style.width = this._width;
            this._flushQueuedAttributes();
            // supports themes or custom properties set in CSS
            this._styling(this, this);
            this.afterContentInit();
        }
        afterContentInit() {
        }
        _enqueueSetAttribute(attrName, attrValue) {
            this._queuedSetAttributes.push({ attrName: attrName, attrValue: attrValue });
        }
        _flushQueuedAttributes() {
            this._settingAttributes = true;
            for (let i = 0; i < this._queuedSetAttributes.length; i++) {
                this.setAttribute(this._queuedSetAttributes[i].attrName, this._queuedSetAttributes[i].attrValue);
            }
            this._settingAttributes = false;
            this._queuedSetAttributes.length = 0;
        }
        _a(attrName, attrValue) {
            if (this._updatingFromAttribute) {
                return;
            }
            if (attrValue) {
                attrValue = attrValue.toString();
            }
            this._settingAttributes = true;
            attrName = toSpinal(attrName);
            if (this._attached) {
                this.setAttribute(attrName, attrValue);
            }
            else {
                this._enqueueSetAttribute(attrName, attrValue);
            }
            this._settingAttributes = false;
        }
        static get observedAttributes() {
            if (IgcPopupComponent._observedAttributesIgcPopupComponent == null) {
                let names = getAllPropertyNames(IgcPopupComponent);
                for (let i = 0; i < names.length; i++) {
                    names[i] = toSpinal(names[i]);
                }
                IgcPopupComponent._observedAttributesIgcPopupComponent = names;
            }
            return IgcPopupComponent._observedAttributesIgcPopupComponent;
        }
        attributeChangedCallback(name, oldValue, newValue) {
            if (this._settingAttributes) {
                return;
            }
            let setName = fromSpinal(name);
            this._updatingFromAttribute = true;
            this[setName] = newValue;
            this._updatingFromAttribute = false;
        }
        static register() {
            if (!IgcPopupComponent._isElementRegistered) {
                IgcPopupComponent._isElementRegistered = true;
                RegisterElementHelper.registerElement(IgcPopupComponent.htmlTagName, IgcPopupComponent);
            }
        }
        get isFocusable() {
            return this.i.as;
        }
        set isFocusable(v) {
            this.i.as = ensureBool(v);
            this._a("isFocusable", this.i.as);
        }
        /**
         * Gets or sets the duration of the Popup and Closeup animations in milliseconds.
        */
        get animationDuration() {
            return this.i.a9;
        }
        set animationDuration(v) {
            this.i.a9 = +v;
            this._a("animationDuration", this.i.a9);
        }
        /**
         * Gets or sets whether the popup and closeup animations are animated.
        */
        get animationEnabled() {
            return this.i.am;
        }
        set animationEnabled(v) {
            this.i.am = ensureBool(v);
            this._a("animationEnabled", this.i.am);
        }
        /**
         * Gets or sets the type of animation to use when the popup opens and closes.
        */
        get animationType() {
            return this.i.p;
        }
        set animationType(v) {
            this.i.p = ensureEnum(PopupAnimationType_$type, v);
            this._a("animationType", enumToString(PopupAnimationType_$type, this.i.p));
        }
        /**
         * Gets or sets the elevation to use for the button regardless of type.
        */
        get elevation() {
            return this.i.bb;
        }
        set elevation(v) {
            this.i.bb = +v;
            this._a("elevation", this.i.bb);
        }
        /**
         * Gets or sets the elevation of the border shadow
        */
        get actualElevation() {
            return this.i.a7;
        }
        set actualElevation(v) {
            this.i.a7 = +v;
            this._a("actualElevation", this.i.a7);
        }
        /**
         * Gets / sets the isShown state of the panel.
        */
        get isShown() {
            return this.i.ax;
        }
        /**
         * Gets or sets the color to use for the check mark when the checkbox is checked.
        */
        get actualUmbraShadowColor() {
            return brushToString(this.i.cz);
        }
        set actualUmbraShadowColor(v) {
            this.i.cz = stringToBrush(v);
            this._a("actualUmbraShadowColor", brushToString(this.i.cz));
        }
        /**
         * Gets or sets the color to use for the check mark when the checkbox is checked.
        */
        get actualPenumbraShadowColor() {
            return brushToString(this.i.cy);
        }
        set actualPenumbraShadowColor(v) {
            this.i.cy = stringToBrush(v);
            this._a("actualPenumbraShadowColor", brushToString(this.i.cy));
        }
        /**
         * Gets or sets the color to use for the check mark when the checkbox is checked.
        */
        get actualAmbientShadowColor() {
            return brushToString(this.i.cx);
        }
        set actualAmbientShadowColor(v) {
            this.i.cx = stringToBrush(v);
            this._a("actualAmbientShadowColor", brushToString(this.i.cx));
        }
        /**
         * Gets or sets the background color of the popup.
        */
        get background() {
            return brushToString(this.i.c0);
        }
        set background(v) {
            this.i.c0 = stringToBrush(v);
            this._a("background", brushToString(this.i.c0));
        }
        get cornerRadius() {
            return this.i.a1;
        }
        set cornerRadius(v) {
            this.i.a1 = +v;
            this._a("cornerRadius", this.i.a1);
        }
        /**
         * Gets or sets whether to disable hit testing on the popup.
        */
        get isHitTestVisible() {
            return this.i.at;
        }
        set isHitTestVisible(v) {
            this.i.at = ensureBool(v);
            this._a("isHitTestVisible", this.i.at);
        }
        /**
         * Gets or sets whether to disable hit testing during the open and close animations.
        */
        get disableHitTestDuringAnimation() {
            return this.i.ao;
        }
        set disableHitTestDuringAnimation(v) {
            this.i.ao = ensureBool(v);
            this._a("disableHitTestDuringAnimation", this.i.ao);
        }
        /**
         * Gets or sets whether to show a pointer off the side of the popup towards the popup target.
        */
        get isPointerEnabled() {
            return this.i.au;
        }
        set isPointerEnabled(v) {
            this.i.au = ensureBool(v);
            this._a("isPointerEnabled", this.i.au);
        }
        /**
         * Gets or sets the pointer position.
        */
        get pointerPosition() {
            return this.i.v;
        }
        set pointerPosition(v) {
            this.i.v = ensureEnum(PopupPointerPosition_$type, v);
            this._a("pointerPosition", enumToString(PopupPointerPosition_$type, this.i.v));
        }
        /**
         * Gets or sets the pointer background color.
        */
        get pointerBackground() {
            return brushToString(this.i.c4);
        }
        set pointerBackground(v) {
            this.i.c4 = stringToBrush(v);
            this._a("pointerBackground", brushToString(this.i.c4));
        }
        /**
         * Gets or sets the pointer size.
        */
        get pointerSize() {
            return this.i.a4;
        }
        set pointerSize(v) {
            this.i.a4 = +v;
            this._a("pointerSize", this.i.a4);
        }
        /**
         * Gets whether the popup is in the middle of opening or not.
        */
        get isShowing() {
            return this.i.av;
        }
        set isShowing(v) {
            this.i.av = ensureBool(v);
            this._a("isShowing", this.i.av);
        }
        /**
         * Gets whether the popup is in the middle of closing or not.
        */
        get isClosing() {
            return this.i.aq;
        }
        set isClosing(v) {
            this.i.aq = ensureBool(v);
            this._a("isClosing", this.i.aq);
        }
        /**
         * Indicates that the popup will position itself relative to the window instead of the document.
        */
        get isFixed() {
            return this.i.ar;
        }
        set isFixed(v) {
            this.i.ar = ensureBool(v);
            this._a("isFixed", this.i.ar);
        }
        /**
         * Indicates that the popup will place itself into the browser top layer.
        */
        get useTopLayer() {
            return this.i.ay;
        }
        set useTopLayer(v) {
            this.i.ay = ensureBool(v);
            this._a("useTopLayer", this.i.ay);
        }
        findByName(name) {
            if (this.findEphemera) {
                if (name && name.indexOf("@@e:") == 0) {
                    return this.findEphemera(name);
                }
            }
            return null;
        }
        get hasUserValues() {
            return this._hasUserValues;
        }
        __m(propertyName) {
            if (!this._inStyling) {
                this._hasUserValues.add(propertyName);
            }
        }
        _styling(container, component, parent) {
            if (this._inStyling) {
                return;
            }
            this._inStyling = true;
            this._stylingContainer = container;
            this._stylingParent = component;
            let genericPrefix = "";
            let typeName = this.i.$type.name;
            if (typeName.indexOf("Xam") === 0) {
                typeName = typeName.substring(3);
            }
            genericPrefix = toSpinal("PopupComponent");
            let additionalPrefixes = [];
            let prefix = toSpinal(typeName);
            additionalPrefixes.push(prefix + "-");
            let b = this.i.$type.baseType;
            while (b && b.name != "Object" &&
                b.name != "Base" &&
                b.name != "Control" &&
                b.Name != "DependencyObject" &&
                b.Name != "FrameworkElement") {
                typeName = b.name;
                if (typeName.indexOf("Xam") === 0) {
                    typeName = typeName.substring(3);
                }
                let basePrefix = toSpinal(typeName);
                additionalPrefixes.push(basePrefix + "-");
                b = b.baseType;
            }
            if (parent) {
                let parentTypeName = parent.i.$type.name;
                if (parentTypeName.indexOf("Xam") === 0) {
                    parentTypeName = parentTypeName.substring(3);
                }
                let parentPrefix = toSpinal(parentTypeName);
                additionalPrefixes.push(parentPrefix + "-" + genericPrefix + "-");
                additionalPrefixes.push(parentPrefix + "-" + prefix + "-");
            }
            initializePropertiesFromCss(container, this, genericPrefix + "-", this.hasUserValues, false, additionalPrefixes);
            if (this._otherStyling) {
                this._otherStyling(container, component, parent);
            }
            this._inStyling = false;
        }
        /**
     * Closes the popup.
    
    */
        close() {
            this.i.b6();
        }
        showRelativeToExclusionRect(exclusionRect, popupDirection, popupAlignment) {
            this.i.showRelativeToExclusionRect(toRect(exclusionRect), (popupDirection == null ? null : popupDirection), (popupAlignment == null ? null : popupAlignment));
        }
        /**
         * Exports visual information about the current state of the grid.
        
        */
        exportVisualModel() {
            let iv = this.i.bd();
            return (iv);
        }
        /**
         * Returns a serialized copy of the exported visual model
        
        */
        exportSerializedVisualModel() {
            let iv = this.i.bq();
            return (iv);
        }
        get onClosed() {
            return this._onClosed;
        }
        set onClosed(ev) {
            if (this._onClosed_wrapped !== null) {
                this.i.onClosed = delegateRemove(this.i.onClosed, this._onClosed_wrapped);
                this._onClosed_wrapped = null;
                this._onClosed = null;
            }
            this._onClosed = ev;
            this._onClosed_wrapped = (o, e) => {
                let outerArgs = new IgcOnClosedEventArgs();
                outerArgs._provideImplementation(e);
                if (this.beforeOnClosed) {
                    this.beforeOnClosed(this, outerArgs);
                }
                if (this._onClosed) {
                    this._onClosed(this, outerArgs);
                }
            };
            this.i.onClosed = delegateCombine(this.i.onClosed, this._onClosed_wrapped);
            ;
        }
        /**
         * Fires when the popup is shown
        */
        get onPopup() {
            return this._onPopup;
        }
        set onPopup(ev) {
            if (this._onPopup_wrapped !== null) {
                this.i.onPopup = delegateRemove(this.i.onPopup, this._onPopup_wrapped);
                this._onPopup_wrapped = null;
                this._onPopup = null;
            }
            this._onPopup = ev;
            this._onPopup_wrapped = (o, e) => {
                let outerArgs = new IgcOnPopupEventArgs();
                outerArgs._provideImplementation(e);
                if (this.beforeOnPopup) {
                    this.beforeOnPopup(this, outerArgs);
                }
                if (this._onPopup) {
                    this._onPopup(this, outerArgs);
                }
            };
            this.i.onPopup = delegateCombine(this.i.onPopup, this._onPopup_wrapped);
            ;
        }
        /**
         * Fired when the open popup gains focus.
        */
        get popupGotFocus() {
            return this._popupGotFocus;
        }
        set popupGotFocus(ev) {
            if (this._popupGotFocus_wrapped !== null) {
                this.i.popupGotFocus = delegateRemove(this.i.popupGotFocus, this._popupGotFocus_wrapped);
                this._popupGotFocus_wrapped = null;
                this._popupGotFocus = null;
            }
            this._popupGotFocus = ev;
            this._popupGotFocus_wrapped = (o, e) => {
                let outerArgs = new IgcFocusEventArgs();
                outerArgs._provideImplementation(e);
                if (this.beforePopupGotFocus) {
                    this.beforePopupGotFocus(this, outerArgs);
                }
                if (this._popupGotFocus) {
                    this._popupGotFocus(this, outerArgs);
                }
            };
            this.i.popupGotFocus = delegateCombine(this.i.popupGotFocus, this._popupGotFocus_wrapped);
            ;
        }
        /**
         * Fired when the open popup loses focus.
        */
        get popupLostFocus() {
            return this._popupLostFocus;
        }
        set popupLostFocus(ev) {
            if (this._popupLostFocus_wrapped !== null) {
                this.i.popupLostFocus = delegateRemove(this.i.popupLostFocus, this._popupLostFocus_wrapped);
                this._popupLostFocus_wrapped = null;
                this._popupLostFocus = null;
            }
            this._popupLostFocus = ev;
            this._popupLostFocus_wrapped = (o, e) => {
                let outerArgs = new IgcFocusEventArgs();
                outerArgs._provideImplementation(e);
                if (this.beforePopupLostFocus) {
                    this.beforePopupLostFocus(this, outerArgs);
                }
                if (this._popupLostFocus) {
                    this._popupLostFocus(this, outerArgs);
                }
            };
            this.i.popupLostFocus = delegateCombine(this.i.popupLostFocus, this._popupLostFocus_wrapped);
            ;
        }
        get measuringContentSize() {
            return this._measuringContentSize;
        }
        set measuringContentSize(ev) {
            if (this._measuringContentSize_wrapped !== null) {
                this.i.measuringContentSize = delegateRemove(this.i.measuringContentSize, this._measuringContentSize_wrapped);
                this._measuringContentSize_wrapped = null;
                this._measuringContentSize = null;
            }
            this._measuringContentSize = ev;
            this._measuringContentSize_wrapped = (o, e) => {
                let outerArgs = new IgcPopupMeasuringContentSizeEventArgs();
                outerArgs._provideImplementation(e);
                if (this.beforeMeasuringContentSize) {
                    this.beforeMeasuringContentSize(this, outerArgs);
                }
                if (this._measuringContentSize) {
                    this._measuringContentSize(this, outerArgs);
                }
            };
            this.i.measuringContentSize = delegateCombine(this.i.measuringContentSize, this._measuringContentSize_wrapped);
            ;
        }
    }
    IgcPopupComponent._observedAttributesIgcPopupComponent = null;
    IgcPopupComponent.htmlTagName = "igc-popup";
    IgcPopupComponent._isElementRegistered = false;
    return IgcPopupComponent;
})();
