import { IgcHTMLElement } from "./igc-html-element";
export let IgcPortalItemComponent = /*@__PURE__*/ (() => {
    class IgcPortalItemComponent extends IgcHTMLElement {
        get portalId() {
            return this._portalId;
        }
        constructor() {
            super();
            this._portalId = 0;
            this._portalId = IgcPortalItemComponent._nextPortalId;
            IgcPortalItemComponent._nextPortalId += 1;
        }
        connectedCallback() {
            if (this.parentElement) {
                if (this.parentElement instanceof IgcPortalEntranceComponent) {
                    let entrance = this.parentElement;
                    entrance.ensurePortalUpdated();
                }
            }
        }
    }
    IgcPortalItemComponent._nextPortalId = 0;
    return IgcPortalItemComponent;
})();
export class IgcPortalEntranceComponent extends IgcHTMLElement {
    constructor() {
        super(...arguments);
        this._detach = null;
        this._observer = null;
        this._suspendChangeDetection = false;
        this._itemContent = new Map();
    }
    get portalId() {
        return this.getAttribute("portal-id");
    }
    set portalId(value) {
        this.setAttribute("portal-id", value);
    }
    get useShadowMode() {
        return this.getAttribute("use-shadow-mode") == "true";
    }
    set useShadowMode(value) {
        this.setAttribute("use-shadow-mode", value ? "true" : "false");
    }
    get moveOnceMode() {
        return this.getAttribute("move-once-mode") == "true";
    }
    set moveOnceMode(value) {
        this.setAttribute("move-once-mode", value ? "true" : "false");
    }
    connectedCallback() {
        this._detach = PortalChannel.instance().subscribe(this.portalId, (m) => this.onPortalMessage(m));
        PortalChannel.instance().broadcast(PortalMessage.entranceAttaching(this, this.portalId, this.useShadowMode));
        if (this.useShadowMode) {
        }
        else {
            this._observer = new MutationObserver((mutations) => {
                if (!this._suspendChangeDetection) {
                    this.ensurePortalUpdated();
                }
            });
            this._observer.observe(this, { childList: true, subtree: true });
        }
        this.ensurePortalUpdated();
    }
    getPortalContent() {
        let useMoveOnce = this.moveOnceMode;
        let children = Array.from(this.children);
        let items = children.filter((c) => c instanceof IgcPortalItemComponent);
        if (useMoveOnce) {
            items = children;
        }
        let ret = [];
        let seen = new Set();
        for (let i = 0; i < items.length; i++) {
            let currItem = items[i];
            let prevContent = null;
            if (!useMoveOnce) {
                let portalItem = currItem;
                seen.add(portalItem.portalId);
                if (this._itemContent.has(portalItem.portalId)) {
                    prevContent = this._itemContent.get(portalItem.portalId);
                }
            }
            let itemChild = useMoveOnce ? currItem : (currItem.children.length > 0 ? currItem.children[0] : null);
            if (!useMoveOnce) {
                let portalItem = currItem;
                if (itemChild && itemChild != prevContent) {
                    this._itemContent.set(portalItem.portalId, itemChild);
                }
                else {
                    itemChild = prevContent;
                }
                if (itemChild) {
                    itemChild.___portalItemId = portalItem.portalId;
                }
            }
            if (itemChild) {
                ret.push(itemChild);
            }
        }
        let toRemove = [];
        for (let key of this._itemContent.keys()) {
            if (!seen.has(key)) {
                toRemove.push(key);
            }
        }
        for (let key of toRemove) {
            this._itemContent.delete(key);
        }
        return ret;
    }
    ensurePortalUpdated() {
        this._suspendChangeDetection = true;
        PortalChannel.instance().broadcast(PortalMessage.entranceSlotContentUpdated(this, this.portalId, this.getPortalContent()));
        this._suspendChangeDetection = false;
    }
    disconnectedCallback() {
        PortalChannel.instance().broadcast(PortalMessage.entranceDetaching(this, this.portalId));
        if (this._detach) {
            this._detach();
        }
        if (this.useShadowMode) {
        }
        else {
            if (this._observer) {
                this._observer.disconnect();
                this._observer = null;
            }
        }
    }
    onPortalMessage(m) {
        if (m.sender === this) {
            return;
        }
        if (m.portalId == this.portalId) {
            switch (m.messageType) {
                case PortalMessageType.ExitAttaching:
                    this.ensurePortalUpdated();
                    break;
            }
        }
    }
}
export class IgcPortalExitComponent extends IgcHTMLElement {
    constructor() {
        super(...arguments);
        this._detach = null;
    }
    get portalId() {
        return this.getAttribute("portal-id");
    }
    set portalId(value) {
        this.setAttribute("portal-id", value);
    }
    get useShadowMode() {
        return this.getAttribute("use-shadow-mode") == "true";
    }
    set useShadowMode(value) {
        this.setAttribute("use-shadow-mode", value ? "true" : "false");
    }
    get useParentAsTarget() {
        return this.getAttribute("use-parent-as-target") == "true";
    }
    set useParentAsTarget(value) {
        this.setAttribute("use-parent-as-target", value ? "true" : "false");
    }
    get moveOnceMode() {
        return this.getAttribute("move-once-mode") == "true";
    }
    set moveOnceMode(value) {
        this.setAttribute("move-once-mode", value ? "true" : "false");
    }
    connectedCallback() {
        this._detach = PortalChannel.instance().subscribe(this.portalId, (m) => this.onPortalMessage(m));
        PortalChannel.instance().broadcast(PortalMessage.exitAttaching(this, this.portalId, this.useShadowMode));
    }
    disconnectedCallback() {
        PortalChannel.instance().broadcast(PortalMessage.exitDetaching(this, this.portalId));
        if (this._detach) {
            this._detach();
        }
    }
    onPortalMessage(m) {
        if (m.sender === this) {
            return;
        }
        if (m.portalId == this.portalId) {
            switch (m.messageType) {
                case PortalMessageType.EntranceSlotContentUpdated:
                    this.onSlotContentChanging(m);
                    break;
            }
        }
    }
    onSlotContentChanging(m) {
        let useMoveOnce = this.moveOnceMode;
        if (this.useShadowMode) {
        }
        else {
            let target = this;
            if (this.useParentAsTarget && this.parentElement) {
                target = this.parentElement;
            }
            if (!useMoveOnce) {
                for (let i = target.children.length - 1; i >= 0; i--) {
                    if (target.children[i] === this) {
                        continue;
                    }
                    target.children[i].remove();
                }
            }
            if (m.content) {
                let arr = m.content;
                for (let i = 0; i < arr.length; i++) {
                    target.append(arr[i]);
                }
            }
        }
    }
}
let PortalChannel = /*@__PURE__*/ (() => {
    class PortalChannel {
        constructor() {
            this._subscribers = new Map();
        }
        static instance() {
            if (this._instance == null) {
                this._instance = new PortalChannel();
            }
            return this._instance;
        }
        subscribe(portalId, onMessage) {
            if (!this._subscribers.has(portalId)) {
                this._subscribers.set(portalId, []);
            }
            this._subscribers.get(portalId).push(onMessage);
            return () => {
                if (this._subscribers.has(portalId)) {
                    let ind = this._subscribers.get(portalId).indexOf(onMessage);
                    this._subscribers.get(portalId).splice(ind, 1);
                    if (this._subscribers.get(portalId).length == 0) {
                        this._subscribers.delete(portalId);
                    }
                }
            };
        }
        broadcast(m) {
            if (!this._subscribers.has(m.portalId)) {
                return;
            }
            let subs = this._subscribers.get(m.portalId).slice();
            for (let i = 0; i < subs.length; i++) {
                subs[i](m);
            }
        }
    }
    PortalChannel._instance = null;
    return PortalChannel;
})();
var PortalMessageType = /*@__PURE__*/ (function (PortalMessageType) {
    PortalMessageType[PortalMessageType["EntranceAttaching"] = 0] = "EntranceAttaching";
    PortalMessageType[PortalMessageType["EntranceDetaching"] = 1] = "EntranceDetaching";
    PortalMessageType[PortalMessageType["ExitAttaching"] = 2] = "ExitAttaching";
    PortalMessageType[PortalMessageType["ExitDetaching"] = 3] = "ExitDetaching";
    PortalMessageType[PortalMessageType["EntranceSlotContentUpdated"] = 4] = "EntranceSlotContentUpdated";
    return PortalMessageType;
})({});
class PortalMessage {
    constructor() {
        this.useShadowMode = false;
        this.messageType = PortalMessageType.EntranceAttaching;
        this.portalId = "";
        this.content = null;
        this.sender = null;
    }
    static entranceAttaching(sender, portalId, useShadowMode) {
        let m = new PortalMessage();
        m.messageType = PortalMessageType.EntranceAttaching;
        m.sender = sender;
        m.portalId = portalId;
        m.useShadowMode = useShadowMode;
        return m;
    }
    static entranceDetaching(sender, portalId) {
        let m = new PortalMessage();
        m.messageType = PortalMessageType.EntranceDetaching;
        m.sender = sender;
        m.portalId = portalId;
        return m;
    }
    static exitAttaching(sender, portalId, useShadowMode) {
        let m = new PortalMessage();
        m.messageType = PortalMessageType.ExitAttaching;
        m.sender = sender;
        m.portalId = portalId;
        m.useShadowMode = useShadowMode;
        return m;
    }
    static exitDetaching(sender, portalId) {
        let m = new PortalMessage();
        m.messageType = PortalMessageType.ExitDetaching;
        m.sender = sender;
        m.portalId = portalId;
        return m;
    }
    static entranceSlotContentUpdated(sender, portalId, content) {
        let m = new PortalMessage();
        m.messageType = PortalMessageType.EntranceSlotContentUpdated;
        m.sender = sender;
        m.portalId = portalId;
        m.content = content;
        return m;
    }
}
export class IgcPortalModule {
    static register() {
        window.customElements.define("igc-portal-entrance", IgcPortalEntranceComponent);
        window.customElements.define("igc-portal-exit", IgcPortalExitComponent);
        window.customElements.define("igc-portal-item", IgcPortalItemComponent);
    }
}
