import { __extends, __values } from "tslib";
import { toRect, ensureEnum, brushToString, stringToBrush, ensureBool, initializePropertiesFromCss, NamePatcher, toSpinal, enumToString, getAllPropertyNames, fromSpinal } from "./componentUtil";
import { WebComponentRenderer, PortalManager } from "./web-component-renderer";
import { delegateCombine, delegateRemove } from "./type";
import { RegisterElementHelper } from "./module-manager";
import { Popup } from "./Popup";
import { IgcOnPopupEventArgs } from './igc-on-popup-event-args';
import { IgcOnClosedEventArgs } from './igc-on-closed-event-args';
import { IgcFocusEventArgs } from './igc-focus-event-args';
import { PopupAnimationType_$type } from "./PopupAnimationType";
import { PopupPointerPosition_$type } from "./PopupPointerPosition";
import { IgcHTMLElement } from "./igc-html-element";
import { IgcPopupMeasuringContentSizeEventArgs } from './igc-popup-measuring-content-size-event-args';
var IgcPopupComponent = /** @class */ /*@__PURE__*/ (function (_super) {
    __extends(IgcPopupComponent, _super);
    function IgcPopupComponent() {
        var _this = _super.call(this) || this;
        _this._implementation = null;
        _this._disconnected = false;
        _this._settingAttributes = false;
        _this._attached = false;
        _this._queuedSetAttributes = [];
        _this._updatingFromAttribute = false;
        _this.__p = null;
        _this._hasUserValues = new Set();
        _this._stylingContainer = null;
        _this._stylingParent = null;
        _this._inStyling = false;
        _this._onClosed = null;
        _this._onClosed_wrapped = null;
        _this._onPopup = null;
        _this._onPopup_wrapped = null;
        _this._popupGotFocus = null;
        _this._popupGotFocus_wrapped = null;
        _this._popupLostFocus = null;
        _this._popupLostFocus_wrapped = null;
        _this._measuringContentSize = null;
        _this._measuringContentSize_wrapped = null;
        if (_this._styling) {
            NamePatcher.ensureStylablePatched(Object.getPrototypeOf(_this));
        }
        _this._implementation = _this.createImplementation();
        _this._portalManager = new PortalManager("popupContent");
        _this._webComponentRenderer = new WebComponentRenderer(_this, document, true, {}, _this._portalManager);
        _this._mainDiv = _this._webComponentRenderer.createElement("div");
        _this._webComponentRenderer.updateRoot(_this._mainDiv);
        _this.i.provideRenderer(_this._webComponentRenderer);
        var mut = new MutationObserver(function (list) {
            var e_1, _b;
            try {
                for (var list_1 = __values(list), list_1_1 = list_1.next(); !list_1_1.done; list_1_1 = list_1.next()) {
                    var mutation = list_1_1.value;
                    if (mutation.type == 'childList') {
                        _this._onChildrenChanged();
                    }
                }
            }
            catch (e_1_1) {
                e_1 = { error: e_1_1 };
            }
            finally {
                try {
                    if (list_1_1 && !list_1_1.done && (_b = list_1.return))
                        _b.call(list_1);
                }
                finally {
                    if (e_1)
                        throw e_1.error;
                }
            }
        });
        mut.observe(_this, {
            childList: true
        });
        return _this;
    }
    Object.defineProperty(IgcPopupComponent.prototype, "height", {
        get: function () {
            return this._height;
        },
        set: function (value) {
            this._height = value;
            this.style.height = value;
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(IgcPopupComponent.prototype, "width", {
        get: function () {
            return this._width;
        },
        set: function (value) {
            this._width = value;
            this.style.width = value;
        },
        enumerable: false,
        configurable: true
    });
    IgcPopupComponent.prototype._onChildrenChanged = function () {
        var children = [];
        for (var i = 0; i < this.childNodes.length; i++) {
            var child = this.childNodes[i];
            if (child !== this._mainDiv.getNativeElement()) {
                children.push(child);
            }
        }
        this._portalManager.onChildrenChanged(children);
    };
    // supports angular themes or custom properties set in CSS
    IgcPopupComponent.prototype.updateStyle = function () {
        this._styling(this, this);
    };
    IgcPopupComponent.prototype.destroy = function () {
        this._implementation.destroy();
        this._webComponentRenderer.destroy();
    };
    IgcPopupComponent.prototype.createImplementation = function () {
        return new Popup();
    };
    Object.defineProperty(IgcPopupComponent.prototype, "i", {
        get: function () {
            return this._implementation;
        },
        enumerable: false,
        configurable: true
    });
    IgcPopupComponent.prototype.disconnectedCallback = function () {
        this._disconnected = true;
    };
    IgcPopupComponent.prototype.connectedCallback = function () {
        if (this._disconnected) {
            this._disconnected = false;
            return;
        }
        this.classList.add("ig-popup");
        this.classList.add("igr-popup");
        this.appendChild(this._mainDiv.getNativeElement());
        this._attached = true;
        this.style.display = "block";
        this.style.height = this._height;
        this.style.width = this._width;
        this._flushQueuedAttributes();
        // supports themes or custom properties set in CSS
        this._styling(this, this);
        this.afterContentInit();
    };
    IgcPopupComponent.prototype.afterContentInit = function () {
    };
    IgcPopupComponent.prototype._enqueueSetAttribute = function (attrName, attrValue) {
        this._queuedSetAttributes.push({ attrName: attrName, attrValue: attrValue });
    };
    IgcPopupComponent.prototype._flushQueuedAttributes = function () {
        this._settingAttributes = true;
        for (var i = 0; i < this._queuedSetAttributes.length; i++) {
            this.setAttribute(this._queuedSetAttributes[i].attrName, this._queuedSetAttributes[i].attrValue);
        }
        this._settingAttributes = false;
        this._queuedSetAttributes.length = 0;
    };
    IgcPopupComponent.prototype._a = function (attrName, attrValue) {
        if (this._updatingFromAttribute) {
            return;
        }
        if (attrValue) {
            attrValue = attrValue.toString();
        }
        this._settingAttributes = true;
        attrName = toSpinal(attrName);
        if (this._attached) {
            this.setAttribute(attrName, attrValue);
        }
        else {
            this._enqueueSetAttribute(attrName, attrValue);
        }
        this._settingAttributes = false;
    };
    Object.defineProperty(IgcPopupComponent, "observedAttributes", {
        get: function () {
            if (IgcPopupComponent._observedAttributesIgcPopupComponent == null) {
                var names = getAllPropertyNames(IgcPopupComponent);
                for (var i = 0; i < names.length; i++) {
                    names[i] = toSpinal(names[i]);
                }
                IgcPopupComponent._observedAttributesIgcPopupComponent = names;
            }
            return IgcPopupComponent._observedAttributesIgcPopupComponent;
        },
        enumerable: false,
        configurable: true
    });
    IgcPopupComponent.prototype.attributeChangedCallback = function (name, oldValue, newValue) {
        if (this._settingAttributes) {
            return;
        }
        var setName = fromSpinal(name);
        this._updatingFromAttribute = true;
        this[setName] = newValue;
        this._updatingFromAttribute = false;
    };
    IgcPopupComponent.register = function () {
        if (!IgcPopupComponent._isElementRegistered) {
            IgcPopupComponent._isElementRegistered = true;
            RegisterElementHelper.registerElement(IgcPopupComponent.htmlTagName, IgcPopupComponent);
        }
    };
    Object.defineProperty(IgcPopupComponent.prototype, "isFocusable", {
        get: function () {
            return this.i.as;
        },
        set: function (v) {
            this.i.as = ensureBool(v);
            this._a("isFocusable", this.i.as);
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(IgcPopupComponent.prototype, "animationDuration", {
        /**
         * Gets or sets the duration of the Popup and Closeup animations in milliseconds.
        */
        get: function () {
            return this.i.a9;
        },
        set: function (v) {
            this.i.a9 = +v;
            this._a("animationDuration", this.i.a9);
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(IgcPopupComponent.prototype, "animationEnabled", {
        /**
         * Gets or sets whether the popup and closeup animations are animated.
        */
        get: function () {
            return this.i.am;
        },
        set: function (v) {
            this.i.am = ensureBool(v);
            this._a("animationEnabled", this.i.am);
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(IgcPopupComponent.prototype, "animationType", {
        /**
         * Gets or sets the type of animation to use when the popup opens and closes.
        */
        get: function () {
            return this.i.p;
        },
        set: function (v) {
            this.i.p = ensureEnum(PopupAnimationType_$type, v);
            this._a("animationType", enumToString(PopupAnimationType_$type, this.i.p));
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(IgcPopupComponent.prototype, "elevation", {
        /**
         * Gets or sets the elevation to use for the button regardless of type.
        */
        get: function () {
            return this.i.bb;
        },
        set: function (v) {
            this.i.bb = +v;
            this._a("elevation", this.i.bb);
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(IgcPopupComponent.prototype, "actualElevation", {
        /**
         * Gets or sets the elevation of the border shadow
        */
        get: function () {
            return this.i.a7;
        },
        set: function (v) {
            this.i.a7 = +v;
            this._a("actualElevation", this.i.a7);
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(IgcPopupComponent.prototype, "isShown", {
        /**
         * Gets / sets the isShown state of the panel.
        */
        get: function () {
            return this.i.ax;
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(IgcPopupComponent.prototype, "actualUmbraShadowColor", {
        /**
         * Gets or sets the color to use for the check mark when the checkbox is checked.
        */
        get: function () {
            return brushToString(this.i.cz);
        },
        set: function (v) {
            this.i.cz = stringToBrush(v);
            this._a("actualUmbraShadowColor", brushToString(this.i.cz));
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(IgcPopupComponent.prototype, "actualPenumbraShadowColor", {
        /**
         * Gets or sets the color to use for the check mark when the checkbox is checked.
        */
        get: function () {
            return brushToString(this.i.cy);
        },
        set: function (v) {
            this.i.cy = stringToBrush(v);
            this._a("actualPenumbraShadowColor", brushToString(this.i.cy));
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(IgcPopupComponent.prototype, "actualAmbientShadowColor", {
        /**
         * Gets or sets the color to use for the check mark when the checkbox is checked.
        */
        get: function () {
            return brushToString(this.i.cx);
        },
        set: function (v) {
            this.i.cx = stringToBrush(v);
            this._a("actualAmbientShadowColor", brushToString(this.i.cx));
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(IgcPopupComponent.prototype, "background", {
        /**
         * Gets or sets the background color of the popup.
        */
        get: function () {
            return brushToString(this.i.c0);
        },
        set: function (v) {
            this.i.c0 = stringToBrush(v);
            this._a("background", brushToString(this.i.c0));
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(IgcPopupComponent.prototype, "cornerRadius", {
        get: function () {
            return this.i.a1;
        },
        set: function (v) {
            this.i.a1 = +v;
            this._a("cornerRadius", this.i.a1);
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(IgcPopupComponent.prototype, "isHitTestVisible", {
        /**
         * Gets or sets whether to disable hit testing on the popup.
        */
        get: function () {
            return this.i.at;
        },
        set: function (v) {
            this.i.at = ensureBool(v);
            this._a("isHitTestVisible", this.i.at);
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(IgcPopupComponent.prototype, "disableHitTestDuringAnimation", {
        /**
         * Gets or sets whether to disable hit testing during the open and close animations.
        */
        get: function () {
            return this.i.ao;
        },
        set: function (v) {
            this.i.ao = ensureBool(v);
            this._a("disableHitTestDuringAnimation", this.i.ao);
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(IgcPopupComponent.prototype, "isPointerEnabled", {
        /**
         * Gets or sets whether to show a pointer off the side of the popup towards the popup target.
        */
        get: function () {
            return this.i.au;
        },
        set: function (v) {
            this.i.au = ensureBool(v);
            this._a("isPointerEnabled", this.i.au);
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(IgcPopupComponent.prototype, "pointerPosition", {
        /**
         * Gets or sets the pointer position.
        */
        get: function () {
            return this.i.v;
        },
        set: function (v) {
            this.i.v = ensureEnum(PopupPointerPosition_$type, v);
            this._a("pointerPosition", enumToString(PopupPointerPosition_$type, this.i.v));
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(IgcPopupComponent.prototype, "pointerBackground", {
        /**
         * Gets or sets the pointer background color.
        */
        get: function () {
            return brushToString(this.i.c4);
        },
        set: function (v) {
            this.i.c4 = stringToBrush(v);
            this._a("pointerBackground", brushToString(this.i.c4));
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(IgcPopupComponent.prototype, "pointerSize", {
        /**
         * Gets or sets the pointer size.
        */
        get: function () {
            return this.i.a4;
        },
        set: function (v) {
            this.i.a4 = +v;
            this._a("pointerSize", this.i.a4);
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(IgcPopupComponent.prototype, "isShowing", {
        /**
         * Gets whether the popup is in the middle of opening or not.
        */
        get: function () {
            return this.i.av;
        },
        set: function (v) {
            this.i.av = ensureBool(v);
            this._a("isShowing", this.i.av);
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(IgcPopupComponent.prototype, "isClosing", {
        /**
         * Gets whether the popup is in the middle of closing or not.
        */
        get: function () {
            return this.i.aq;
        },
        set: function (v) {
            this.i.aq = ensureBool(v);
            this._a("isClosing", this.i.aq);
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(IgcPopupComponent.prototype, "isFixed", {
        /**
         * Indicates that the popup will position itself relative to the window instead of the document.
        */
        get: function () {
            return this.i.ar;
        },
        set: function (v) {
            this.i.ar = ensureBool(v);
            this._a("isFixed", this.i.ar);
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(IgcPopupComponent.prototype, "useTopLayer", {
        /**
         * Indicates that the popup will place itself into the browser top layer.
        */
        get: function () {
            return this.i.ay;
        },
        set: function (v) {
            this.i.ay = ensureBool(v);
            this._a("useTopLayer", this.i.ay);
        },
        enumerable: false,
        configurable: true
    });
    IgcPopupComponent.prototype.findByName = function (name) {
        if (this.findEphemera) {
            if (name && name.indexOf("@@e:") == 0) {
                return this.findEphemera(name);
            }
        }
        return null;
    };
    Object.defineProperty(IgcPopupComponent.prototype, "hasUserValues", {
        get: function () {
            return this._hasUserValues;
        },
        enumerable: false,
        configurable: true
    });
    IgcPopupComponent.prototype.__m = function (propertyName) {
        if (!this._inStyling) {
            this._hasUserValues.add(propertyName);
        }
    };
    IgcPopupComponent.prototype._styling = function (container, component, parent) {
        if (this._inStyling) {
            return;
        }
        this._inStyling = true;
        this._stylingContainer = container;
        this._stylingParent = component;
        var genericPrefix = "";
        var typeName = this.i.$type.name;
        if (typeName.indexOf("Xam") === 0) {
            typeName = typeName.substring(3);
        }
        genericPrefix = toSpinal("PopupComponent");
        var additionalPrefixes = [];
        var prefix = toSpinal(typeName);
        additionalPrefixes.push(prefix + "-");
        var b = this.i.$type.baseType;
        while (b && b.name != "Object" &&
            b.name != "Base" &&
            b.name != "Control" &&
            b.Name != "DependencyObject" &&
            b.Name != "FrameworkElement") {
            typeName = b.name;
            if (typeName.indexOf("Xam") === 0) {
                typeName = typeName.substring(3);
            }
            var basePrefix = toSpinal(typeName);
            additionalPrefixes.push(basePrefix + "-");
            b = b.baseType;
        }
        if (parent) {
            var parentTypeName = parent.i.$type.name;
            if (parentTypeName.indexOf("Xam") === 0) {
                parentTypeName = parentTypeName.substring(3);
            }
            var parentPrefix = toSpinal(parentTypeName);
            additionalPrefixes.push(parentPrefix + "-" + genericPrefix + "-");
            additionalPrefixes.push(parentPrefix + "-" + prefix + "-");
        }
        initializePropertiesFromCss(container, this, genericPrefix + "-", this.hasUserValues, false, additionalPrefixes);
        if (this._otherStyling) {
            this._otherStyling(container, component, parent);
        }
        this._inStyling = false;
    };
    /**
 * Closes the popup.

*/
    IgcPopupComponent.prototype.close = function () {
        this.i.b6();
    };
    IgcPopupComponent.prototype.showRelativeToExclusionRect = function (exclusionRect, popupDirection, popupAlignment) {
        this.i.showRelativeToExclusionRect(toRect(exclusionRect), (popupDirection == null ? null : popupDirection), (popupAlignment == null ? null : popupAlignment));
    };
    /**
     * Exports visual information about the current state of the grid.
    
    */
    IgcPopupComponent.prototype.exportVisualModel = function () {
        var iv = this.i.bd();
        return (iv);
    };
    /**
     * Returns a serialized copy of the exported visual model
    
    */
    IgcPopupComponent.prototype.exportSerializedVisualModel = function () {
        var iv = this.i.bq();
        return (iv);
    };
    Object.defineProperty(IgcPopupComponent.prototype, "onClosed", {
        get: function () {
            return this._onClosed;
        },
        set: function (ev) {
            var _this = this;
            if (this._onClosed_wrapped !== null) {
                this.i.onClosed = delegateRemove(this.i.onClosed, this._onClosed_wrapped);
                this._onClosed_wrapped = null;
                this._onClosed = null;
            }
            this._onClosed = ev;
            this._onClosed_wrapped = function (o, e) {
                var outerArgs = new IgcOnClosedEventArgs();
                outerArgs._provideImplementation(e);
                if (_this.beforeOnClosed) {
                    _this.beforeOnClosed(_this, outerArgs);
                }
                if (_this._onClosed) {
                    _this._onClosed(_this, outerArgs);
                }
            };
            this.i.onClosed = delegateCombine(this.i.onClosed, this._onClosed_wrapped);
            ;
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(IgcPopupComponent.prototype, "onPopup", {
        /**
         * Fires when the popup is shown
        */
        get: function () {
            return this._onPopup;
        },
        set: function (ev) {
            var _this = this;
            if (this._onPopup_wrapped !== null) {
                this.i.onPopup = delegateRemove(this.i.onPopup, this._onPopup_wrapped);
                this._onPopup_wrapped = null;
                this._onPopup = null;
            }
            this._onPopup = ev;
            this._onPopup_wrapped = function (o, e) {
                var outerArgs = new IgcOnPopupEventArgs();
                outerArgs._provideImplementation(e);
                if (_this.beforeOnPopup) {
                    _this.beforeOnPopup(_this, outerArgs);
                }
                if (_this._onPopup) {
                    _this._onPopup(_this, outerArgs);
                }
            };
            this.i.onPopup = delegateCombine(this.i.onPopup, this._onPopup_wrapped);
            ;
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(IgcPopupComponent.prototype, "popupGotFocus", {
        /**
         * Fired when the open popup gains focus.
        */
        get: function () {
            return this._popupGotFocus;
        },
        set: function (ev) {
            var _this = this;
            if (this._popupGotFocus_wrapped !== null) {
                this.i.popupGotFocus = delegateRemove(this.i.popupGotFocus, this._popupGotFocus_wrapped);
                this._popupGotFocus_wrapped = null;
                this._popupGotFocus = null;
            }
            this._popupGotFocus = ev;
            this._popupGotFocus_wrapped = function (o, e) {
                var outerArgs = new IgcFocusEventArgs();
                outerArgs._provideImplementation(e);
                if (_this.beforePopupGotFocus) {
                    _this.beforePopupGotFocus(_this, outerArgs);
                }
                if (_this._popupGotFocus) {
                    _this._popupGotFocus(_this, outerArgs);
                }
            };
            this.i.popupGotFocus = delegateCombine(this.i.popupGotFocus, this._popupGotFocus_wrapped);
            ;
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(IgcPopupComponent.prototype, "popupLostFocus", {
        /**
         * Fired when the open popup loses focus.
        */
        get: function () {
            return this._popupLostFocus;
        },
        set: function (ev) {
            var _this = this;
            if (this._popupLostFocus_wrapped !== null) {
                this.i.popupLostFocus = delegateRemove(this.i.popupLostFocus, this._popupLostFocus_wrapped);
                this._popupLostFocus_wrapped = null;
                this._popupLostFocus = null;
            }
            this._popupLostFocus = ev;
            this._popupLostFocus_wrapped = function (o, e) {
                var outerArgs = new IgcFocusEventArgs();
                outerArgs._provideImplementation(e);
                if (_this.beforePopupLostFocus) {
                    _this.beforePopupLostFocus(_this, outerArgs);
                }
                if (_this._popupLostFocus) {
                    _this._popupLostFocus(_this, outerArgs);
                }
            };
            this.i.popupLostFocus = delegateCombine(this.i.popupLostFocus, this._popupLostFocus_wrapped);
            ;
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(IgcPopupComponent.prototype, "measuringContentSize", {
        get: function () {
            return this._measuringContentSize;
        },
        set: function (ev) {
            var _this = this;
            if (this._measuringContentSize_wrapped !== null) {
                this.i.measuringContentSize = delegateRemove(this.i.measuringContentSize, this._measuringContentSize_wrapped);
                this._measuringContentSize_wrapped = null;
                this._measuringContentSize = null;
            }
            this._measuringContentSize = ev;
            this._measuringContentSize_wrapped = function (o, e) {
                var outerArgs = new IgcPopupMeasuringContentSizeEventArgs();
                outerArgs._provideImplementation(e);
                if (_this.beforeMeasuringContentSize) {
                    _this.beforeMeasuringContentSize(_this, outerArgs);
                }
                if (_this._measuringContentSize) {
                    _this._measuringContentSize(_this, outerArgs);
                }
            };
            this.i.measuringContentSize = delegateCombine(this.i.measuringContentSize, this._measuringContentSize_wrapped);
            ;
        },
        enumerable: false,
        configurable: true
    });
    IgcPopupComponent._observedAttributesIgcPopupComponent = null;
    IgcPopupComponent.htmlTagName = "igc-popup";
    IgcPopupComponent._isElementRegistered = false;
    return IgcPopupComponent;
}(IgcHTMLElement));
export { IgcPopupComponent };
